%Projekt: RocketSim

%Funktion position
%Berechnet die Position und Geschwindigkeit
%der Rakete in Abhaengigkeit von der Zeit.

function [t,pos] = position(t, rakete, gam, me, re, phi_start, phi_step, mm, rm, ma) 

    %Startkoordinaten berechnen
    x_start = cos(rakete.start_pos)*re;
    y_start = sin(rakete.start_pos)*re;
    
    %Startwinkel berechnen
    phi_start = phi_start + rakete.start_pos;

    %Differentialgleichung loesen
    [t,pos] = ode45(@posx, t, [x_start,y_start;0,0], odeset('Events', @events));
    
    %Funktion Schub
    %Gibt die Schubkraft der Rakete in Abhaengigkeit der Zeit zur?ck
    function a = Schub(t, flugdaten)
        %Initalisierungswert
        a = [0;0];
        
        %Ist nocht Treibstoff vorhanden?
        if (t <= rakete.t_brennschluss) 
            a = rakete.u*rakete.dm/(rakete.m0 - rakete.dm*t) * [cos(phi_start + phi_step*t); sin(phi_start + phi_step*t)];
        end
        
        %Bremsschub einleiten
        if (rakete.autoflight)
            v1 = sqrt(gam*mm/flugdaten.h_mond);
           bb = 10; 
            if (flugdaten.sx_mond < 0) && (flugdaten.v_res > v1) && (flugdaten.vx > 0) 
                %disp(['Autoflight-Zielgeschwindigkeit: ', num2str(v1)]);
                %disp(['Hoehe Mond: ', num2str(h_mond)]);
                %disp(['Raketen-Geschwindigkeit: ', num2str(v)]);
                a = [-flugdaten.vx; -flugdaten.vy] ./flugdaten.v_res * (flugdaten.v_res - v1);
            end
                
        else
            %Manueller Bremsvorgang
            if (flugdaten.vx > 0) && (flugdaten.h_erde >= rakete.h_brems) && (flugdaten.v_res > rakete.v_brems)
                a = [-flugdaten.vx; -flugdaten.vy] ./flugdaten.v_res * (flugdaten.v_res-rakete.v_brems) ./ 60;
            end
        end
    end

    %Funktion Erdanziehung
    %Berechnet die Erdbeschleunigung in Abhaengigkeit zur Hoehe
    function g = Erdanziehung(flugdaten)
        g = gam*me/(flugdaten.h_erde)^2 * [flugdaten.sx_erde/flugdaten.h_erde; flugdaten.sy_erde/flugdaten.h_erde];
    end

    %Funktion Mondanziehung
    %Berechnet die Mondbeschleunigung in Abhaengigkeit zur Hoehe
    function g = Mondanziehung(flugdaten)
        g = gam*mm/(flugdaten.h_mond)^2 * [-flugdaten.sx_mond/flugdaten.h_mond; flugdaten.sy_mond/flugdaten.h_mond];
    end

    %Differentialgleichungssystem
    %Erklaerung der verwendeten Variablen y
    %y(1): Weg             x-Richtung
    %y(2): Geschwindigkeit x-Richtung
    %y(3): Weg             y-Richtung
    %y(4): Geschwindigkeit y-Richtung
    function [fx] = posx(t,y)
        %Vektoradditionen, Zerlegungen, ...
        flugdaten = ermittleFlugdaten(y, ma);
        
        %Schubbeschleunigung
        a = Schub(t, flugdaten);
        
        %Erdbeschleunigung
        ge = Erdanziehung(flugdaten);
        
        %Mondbeschleunigung
        gm = Mondanziehung(flugdaten);
          
        %Loesungsvektor in der Form
        %[1. Ableitung (Geschwindigkeit); 2. Ableitung (Beschleunigung)]
        dxdt = [y(2); a(1) - ge(1) - gm(1)];
        dydt = [y(4); a(2) - ge(2) - gm(2)];
        
        %Rueckgabe
        fx = [dxdt; dydt];
    end

    %Funktion events
    %Ueberprueft, ob die Rakete auf Erd- oder Mondoberflaeche
    %auftrifft und bricht gegebenfalls die Integration ab
    function [value,isterminal,direction] = events(t,y)
        flugdaten = ermittleFlugdaten(y, ma);
        
        if (flugdaten.h_erde < re) || (flugdaten.h_mond < rm)
            value = 0;    %Absturz
        else
            value = 1;    
        end
        
        isterminal = 1;  
        direction  = 0; 
    end

end